==> react-selectize-0.0.4.tgz: package/src/react-selectize.js <==
(function(){
  var ref$, filter, find, last, map, partition, reverse, sortBy, clamp, findAll, partitionString, remove, onClickOutside, React, div, input, span, SimpleOption, SimpleValue;
  ref$ = require('prelude-ls'), filter = ref$.filter, find = ref$.find, last = ref$.last, map = ref$.map, partition = ref$.partition, reverse = ref$.reverse, sortBy = ref$.sortBy;
  ref$ = require('./prelude-extension'), clamp = ref$.clamp, findAll = ref$.findAll, partitionString = ref$.partitionString, remove = ref$.remove;
  onClickOutside = require('react-onclickoutside');
  React = require('react');
  ref$ = React.DOM, div = ref$.div, input = ref$.input, span = ref$.span;
  SimpleOption = require('./simple-option');
  SimpleValue = require('./simple-value');
  module.exports = React.createClass({
    displayName: 'ReactSelectize',
    mixins: [onClickOutside],
    render: function(){
      var self, ref$, handleClick, handleInputKeyDown, handleOptionClick, handleOptionMouseOver, handleOptionMouseOut, handleRemoveClick, handleResetClick, handleSearchChange, isBelowLimit, ref1$, options, placeholderText, values, maxItems, disabled, style, optionClass, valueClass, focusedOption, open, search, children;
      ref$ = self = this, handleClick = ref$.handleClick, handleInputKeyDown = ref$.handleInputKeyDown, handleOptionClick = ref$.handleOptionClick, handleOptionMouseOver = ref$.handleOptionMouseOver, handleOptionMouseOut = ref$.handleOptionMouseOut, handleRemoveClick = ref$.handleRemoveClick, handleResetClick = ref$.handleResetClick, handleSearchChange = ref$.handleSearchChange, isBelowLimit = ref$.isBelowLimit, ref1$ = ref$.props, options = ref1$.options, placeholderText = ref1$.placeholderText, values = ref1$.values, maxItems = ref1$.maxItems, disabled = ref1$.disabled, style = ref1$.style, optionClass = ref1$.optionClass, valueClass = ref1$.valueClass, ref1$ = ref$.state, focusedOption = ref1$.focusedOption, open = ref1$.open, search = ref1$.search;
      children = [div({
        className: 'control',
        key: 'control'
      }, values.length === 0 && search.length === 0 ? div({
        className: 'placeholder'
      }, placeholderText) : null, map(function(value){
        return React.createElement(valueClass || SimpleValue, import$({
          key: value,
          onRemoveClick: handleRemoveClick.bind(self, value)
        }, find(function(it){
          return it.value === value;
        })(
        options)) || {});
      })(
      values), input(import$({
        disabled: disabled,
        ref: 'search',
        type: 'text',
        value: search,
        onKeyDown: handleInputKeyDown,
        style: {
          width: Math.max(16, search.length * 16)
        }
      }, this.isBelowLimit()
        ? {
          onChange: handleSearchChange
        }
        : {})), div({
        className: 'reset',
        onClick: handleResetClick
      }, '×'), div({
        className: 'arrow'
      }, null))];
      if (open) {
        children.push(div({
          className: 'options',
          key: 'options'
        }, map(function(optionObject){
          var index, value, ref$;
          index = optionObject.index, value = optionObject.value;
          return React.createElement(optionClass || SimpleOption, (ref$ = import$({}, optionObject), ref$.key = value + "", ref$.ref = "option-" + index, ref$.focused = index === focusedOption, ref$.onClick = handleOptionClick.bind(self, index), ref$.onMouseOver = handleOptionMouseOver.bind(self, index), ref$.onMouseOut = handleOptionMouseOut, ref$));
        })(
        this.filterOptions(search))));
      }
      return div({
        className: "multi-select " + (disabled ? 'disabled' : '') + "  " + (open ? 'open' : ''),
        onClick: handleClick,
        style: style
      }, children);
    },
    selectOption: function(index){
      var filteredOptions, option, newOption, value, ref$, ref1$;
      filteredOptions = this.filterOptions(this.state.search);
      if ((option = filteredOptions != null ? filteredOptions[index] : void 8) != null) {
        newOption = option.newOption, value = option.value;
      }
      if (!!newOption) {
        if ((ref$ = this.props) != null) {
          ref$.onOptionsChange([option].concat(this.props.options));
        }
      }
      if (!!value) {
        if ((ref1$ = this.props) != null) {
          ref1$.onChange(this.props.multi
            ? this.props.values.concat(value)
            : [value]);
        }
      }
    },
    removeValue: function(value){
      var option, newOption, ref$, ref1$;
      if ((option = find(function(it){
        return it.value === value;
      })(
      this.props.options)) != null) {
        newOption = option.newOption;
      }
      if (!!newOption) {
        if ((ref$ = this.props) != null) {
          ref$.onOptionsChange(remove(function(it){
            return it.value === value;
          })(
          this.props.options));
        }
      }
      if ((ref1$ = this.props) != null) {
        ref1$.onChange(remove((function(it){
          return it === value;
        }))(
        this.props.values));
      }
      return option;
    },
    reset: function(){
      var ref$;
      return (ref$ = this.props) != null ? ref$.onChange([]) : void 8;
    },
    clearAndFoucs: function(){
      this.setState({
        search: ''
      });
      return this.focus();
    },
    componentDidUpdate: function(){
      var optionElement, ref$, ref1$, parentElement, optionHeight;
      if (this.state.focusedOption === -1) {
        return;
      }
      if ((optionElement = this != null ? (ref$ = this.refs) != null ? (ref1$ = ref$["option-" + this.state.focusedOption]) != null ? ref1$.getDOMNode() : void 8 : void 8 : void 8) != null) {
        parentElement = optionElement.parentElement;
      }
      if (!optionElement) {
        return;
      }
      optionHeight = optionElement.offsetHeight - 1;
      if (optionElement.offsetTop - parentElement.scrollTop > parentElement.offsetHeight) {
        return parentElement.scrollTop = optionElement.offsetTop - parentElement.offsetHeight + optionHeight;
      } else if (optionElement.offsetTop - parentElement.scrollTop + optionHeight < 0) {
        return parentElement.scrollTop = optionElement.offsetTop;
      }
    },
    componentWillReceiveProps: function(newProps){
      console.log("received props");
      return this.setState(this.showOptions(this.state.open, newProps));
    },
    filterOptions: function(search){
      var ref$, optionClass, options, values, result;
      ref$ = this.props, optionClass = ref$.optionClass, options = ref$.options, values = ref$.values;
      result = (optionClass || SimpleOption).filter(filter(function(arg$){
        var value;
        value = arg$.value;
        return !in$(value, values);
      })(
      options), search);
      return map(function(index){
        var ref$;
        return ref$ = result[index], ref$.index = index, ref$;
      })(
      (function(){
        var i$, to$, results$ = [];
        for (i$ = 0, to$ = result.length; i$ < to$; ++i$) {
          results$.push(i$);
        }
        return results$;
      }()));
    },
    focus: function(){
      return this.refs.search.getDOMNode().focus();
    },
    focusAdjacentOption: function(direction){
      var values;
      values = this.props.values;
      return this.setState(import$({
        focusedOption: clamp(this.state.focusedOption + direction, 0, this.filterOptions(this.state.search).length - 1)
      }, this.showOptions(true)));
    },
    getInitialState: function(){
      return {
        focusedOption: 0,
        open: false,
        search: ''
      };
    },
    handleClick: function(){
      this.setState(this.showOptions(true));
      return this.focus();
    },
    handleClickOutside: function(){
      return this.setState({
        open: false
      });
    },
    handleInputKeyDown: function(arg$){
      var which, preventDefault, ref$, label;
      which = arg$.which, preventDefault = arg$.preventDefault;
      switch (ref$ = [which], false) {
      case 8 !== ref$[0]:
        if (this.state.search.length > 0) {
          return;
        }
        label = this.removeValue(last(this.props.values)).label;
        if (!!((ref$ = this.props) != null && ref$.restoreOnBackspace)) {
          this.setState(import$({
            search: label,
            focusedOption: 0
          }, this.showOptions(true)));
        } else {
          this.setState({
            open: false
          });
        }
        break;
      case 13 !== ref$[0]:
        this.selectOption(this.state.focusedOption);
        this.setState({
          focusedOption: -1,
          open: false,
          search: ''
        });
        break;
      case 27 !== ref$[0]:
        if (this.state.open) {
          this.setState({
            open: false
          });
        } else {
          this.reset();
        }
        this.clearAndFoucs();
        break;
      case 38 !== ref$[0]:
        this.focusAdjacentOption(-1);
        break;
      case 40 !== ref$[0]:
        this.focusAdjacentOption(1);
        break;
      default:
        return;
      }
      return false;
    },
    handleOptionClick: function(index){
      var self;
      self = this;
      this.setState({
        open: !this.props.multi
          ? false
          : this.state.open
      }, function(){
        self.selectOption(index);
        return self.clearAndFoucs();
      });
      return false;
    },
    handleOptionMouseOver: function(index){
      return this.setState({
        focusedOption: index
      });
    },
    handleOptionMouseOut: function(){
      return this.setState({
        focusedOption: -1
      });
    },
    handleRemoveClick: function(value){
      this.removeValue(value);
      this.clearAndFoucs();
      return false;
    },
    handleResetClick: function(){
      this.reset();
      this.clearAndFoucs();
      return false;
    },
    handleSearchChange: function(arg$){
      var value, filteredOptions, ref$;
      value = arg$.currentTarget.value;
      filteredOptions = this.filterOptions(value);
      return this.setState({
        focusedOption: filteredOptions.length === 1 || typeof (filteredOptions != null ? (ref$ = filteredOptions[0]) != null ? ref$.newOption : void 8 : void 8) === 'undefined' ? 0 : 1,
        open: this.state.open || value.length > 0,
        search: value
      });
    },
    isBelowLimit: function(props){
      var ref$, maxItems, values;
      if ((ref$ = props || this.props) != null) {
        maxItems = ref$.maxItems, values = ref$.values;
      }
      return typeof maxItems === 'undefined' || values.length < maxItems;
    },
    showOptions: function(show, props){
      var ref$, disabled, multi, options, values;
      ref$ = props || this.props, disabled = ref$.disabled, multi = ref$.multi, options = ref$.options, values = ref$.values;
      return {
        open: (show && !multi) || (show && this.isBelowLimit(props) && !disabled && values.length < options.length)
      };
    }
  });
  function import$(obj, src){
    var own = {}.hasOwnProperty;
    for (var key in src) if (own.call(src, key)) obj[key] = src[key];
    return obj;
  }
  function in$(x, xs){
    var i = -1, l = xs.length >>> 0;
    while (++i < l) if (x === xs[i]) return true;
    return false;
  }
}).call(this);

==> react-selectize-0.0.5.tgz: package/src/react-selectize.js <==
(function(){
  var ref$, filter, find, last, map, partition, reverse, sortBy, clamp, findAll, partitionString, remove, onClickOutside, React, div, input, span, SimpleOption, SimpleValue;
  ref$ = require('prelude-ls'), filter = ref$.filter, find = ref$.find, last = ref$.last, map = ref$.map, partition = ref$.partition, reverse = ref$.reverse, sortBy = ref$.sortBy;
  ref$ = require('./prelude-extension'), clamp = ref$.clamp, findAll = ref$.findAll, partitionString = ref$.partitionString, remove = ref$.remove;
  onClickOutside = require('react-onclickoutside');
  React = require('react');
  ref$ = React.DOM, div = ref$.div, input = ref$.input, span = ref$.span;
  SimpleOption = require('./simple-option');
  SimpleValue = require('./simple-value');
  module.exports = React.createClass({
    displayName: 'ReactSelectize',
    mixins: [onClickOutside],
    render: function(){
      var self, ref$, handleClick, handleInputKeyDown, handleOptionClick, handleOptionMouseOver, handleOptionMouseOut, handleRemoveClick, handleResetClick, handleSearchChange, isBelowLimit, ref1$, addOptions, options, placeholderText, values, maxItems, disabled, style, optionClass, valueClass, focusedOption, open, search, children;
      ref$ = self = this, handleClick = ref$.handleClick, handleInputKeyDown = ref$.handleInputKeyDown, handleOptionClick = ref$.handleOptionClick, handleOptionMouseOver = ref$.handleOptionMouseOver, handleOptionMouseOut = ref$.handleOptionMouseOut, handleRemoveClick = ref$.handleRemoveClick, handleResetClick = ref$.handleResetClick, handleSearchChange = ref$.handleSearchChange, isBelowLimit = ref$.isBelowLimit, ref1$ = ref$.props, addOptions = ref1$.addOptions || false, options = ref1$.options, placeholderText = ref1$.placeholderText, values = ref1$.values, maxItems = ref1$.maxItems, disabled = ref1$.disabled, style = ref1$.style, optionClass = ref1$.optionClass, valueClass = ref1$.valueClass, ref1$ = ref$.state, focusedOption = ref1$.focusedOption, open = ref1$.open, search = ref1$.search;
      children = [div({
        className: 'control',
        key: 'control'
      }, values.length === 0 && search.length === 0 ? div({
        className: 'placeholder'
      }, placeholderText) : null, map(function(value){
        return React.createElement(valueClass || SimpleValue, import$({
          key: value,
          onRemoveClick: handleRemoveClick.bind(self, value)
        }, find(function(it){
          return it.value === value;
        })(
        options)) || {});
      })(
      values), input(import$({
        disabled: disabled,
        ref: 'search',
        type: 'text',
        value: search,
        onKeyDown: handleInputKeyDown,
        style: {
          width: Math.max(16, search.length * 16)
        }
      }, this.isBelowLimit()
        ? {
          onChange: handleSearchChange
        }
        : {})), div({
        className: 'reset',
        onClick: handleResetClick
      }, '×'), div({
        className: 'arrow'
      }, null))];
      if (open) {
        children.push(div({
          className: 'options',
          key: 'options'
        }, map(function(optionObject){
          var index, value, ref$;
          index = optionObject.index, value = optionObject.value;
          return React.createElement(optionClass || SimpleOption, (ref$ = import$({}, optionObject), ref$.addOptions = addOptions, ref$.key = value + "", ref$.ref = "option-" + index, ref$.focused = index === focusedOption, ref$.onClick = handleOptionClick.bind(self, index), ref$.onMouseOver = handleOptionMouseOver.bind(self, index), ref$.onMouseOut = handleOptionMouseOut, ref$));
        })(
        this.filterOptions(search))));
      }
      return div({
        className: "multi-select " + (disabled ? 'disabled' : '') + "  " + (open ? 'open' : ''),
        onClick: handleClick,
        style: style
      }, children);
    },
    selectOption: function(index){
      var filteredOptions, option, newOption, value, ref$, ref1$;
      filteredOptions = this.filterOptions(this.state.search);
      if ((option = filteredOptions != null ? filteredOptions[index] : void 8) != null) {
        newOption = option.newOption, value = option.value;
      }
      if (!!newOption) {
        if ((ref$ = this.props) != null) {
          ref$.onOptionsChange([option].concat(this.props.options));
        }
      }
      if (!!value) {
        if ((ref1$ = this.props) != null) {
          ref1$.onChange(this.props.multi
            ? this.props.values.concat(value)
            : [value]);
        }
      }
    },
    removeValue: function(value){
      var option, newOption, ref$, ref1$;
      if ((option = find(function(it){
        return it.value === value;
      })(
      this.props.options)) != null) {
        newOption = option.newOption;
      }
      if (!!newOption) {
        if ((ref$ = this.props) != null) {
          ref$.onOptionsChange(remove(function(it){
            return it.value === value;
          })(
          this.props.options));
        }
      }
      if ((ref1$ = this.props) != null) {
        ref1$.onChange(remove((function(it){
          return it === value;
        }))(
        this.props.values));
      }
      return option;
    },
    reset: function(){
      var ref$;
      return (ref$ = this.props) != null ? ref$.onChange([]) : void 8;
    },
    clearAndFoucs: function(){
      this.setState({
        search: ''
      });
      return this.focus();
    },
    componentDidUpdate: function(){
      var optionElement, ref$, ref1$, parentElement, optionHeight;
      if (this.state.focusedOption === -1) {
        return;
      }
      if ((optionElement = this != null ? (ref$ = this.refs) != null ? (ref1$ = ref$["option-" + this.state.focusedOption]) != null ? ref1$.getDOMNode() : void 8 : void 8 : void 8) != null) {
        parentElement = optionElement.parentElement;
      }
      if (!optionElement) {
        return;
      }
      optionHeight = optionElement.offsetHeight - 1;
      if (optionElement.offsetTop - parentElement.scrollTop > parentElement.offsetHeight) {
        return parentElement.scrollTop = optionElement.offsetTop - parentElement.offsetHeight + optionHeight;
      } else if (optionElement.offsetTop - parentElement.scrollTop + optionHeight < 0) {
        return parentElement.scrollTop = optionElement.offsetTop;
      }
    },
    componentWillReceiveProps: function(newProps){
      return this.setState(this.showOptions(this.state.open, newProps));
    },
    filterOptions: function(search){
      var ref$, addOptions, optionClass, options, values, result;
      ref$ = this.props, addOptions = ref$.addOptions, optionClass = ref$.optionClass, options = ref$.options, values = ref$.values;
      result = (optionClass || SimpleOption).filter(filter(function(arg$){
        var value;
        value = arg$.value;
        return !in$(value, values);
      })(
      options), search, {
        addOptions: addOptions
      });
      return map(function(index){
        var ref$;
        return ref$ = result[index], ref$.index = index, ref$;
      })(
      (function(){
        var i$, to$, results$ = [];
        for (i$ = 0, to$ = result.length; i$ < to$; ++i$) {
          results$.push(i$);
        }
        return results$;
      }()));
    },
    focus: function(){
      return this.refs.search.getDOMNode().focus();
    },
    focusAdjacentOption: function(direction){
      var values;
      values = this.props.values;
      return this.setState(import$({
        focusedOption: clamp(this.state.focusedOption + direction, 0, this.filterOptions(this.state.search).length - 1)
      }, this.showOptions(true)));
    },
    getInitialState: function(){
      return {
        focusedOption: 0,
        open: false,
        search: ''
      };
    },
    handleClick: function(){
      this.setState(this.showOptions(true));
      return this.focus();
    },
    handleClickOutside: function(){
      return this.setState({
        open: false
      });
    },
    handleInputKeyDown: function(arg$){
      var which, preventDefault, ref$, label;
      which = arg$.which, preventDefault = arg$.preventDefault;
      switch (ref$ = [which], false) {
      case 8 !== ref$[0]:
        if (this.state.search.length > 0) {
          return;
        }
        label = this.removeValue(last(this.props.values)).label;
        if (!!((ref$ = this.props) != null && ref$.restoreOnBackspace)) {
          this.setState(import$({
            search: label,
            focusedOption: 0
          }, this.showOptions(true)));
        } else {
          this.setState({
            open: false
          });
        }
        break;
      case 13 !== ref$[0]:
        this.selectOption(this.state.focusedOption);
        this.setState({
          focusedOption: -1,
          open: false,
          search: ''
        });
        break;
      case 27 !== ref$[0]:
        if (this.state.open) {
          this.setState({
            open: false
          });
        } else {
          this.reset();
        }
        this.clearAndFoucs();
        break;
      case 38 !== ref$[0]:
        this.focusAdjacentOption(-1);
        break;
      case 40 !== ref$[0]:
        this.focusAdjacentOption(1);
        break;
      default:
        return;
      }
      return false;
    },
    handleOptionClick: function(index){
      var self;
      self = this;
      this.setState({
        open: !this.props.multi
          ? false
          : this.state.open
      }, function(){
        self.selectOption(index);
        return self.clearAndFoucs();
      });
      return false;
    },
    handleOptionMouseOver: function(index){
      return this.setState({
        focusedOption: index
      });
    },
    handleOptionMouseOut: function(){
      return this.setState({
        focusedOption: -1
      });
    },
    handleRemoveClick: function(value){
      this.removeValue(value);
      this.clearAndFoucs();
      return false;
    },
    handleResetClick: function(){
      this.reset();
      this.clearAndFoucs();
      return false;
    },
    handleSearchChange: function(arg$){
      var value, filteredOptions, ref$;
      value = arg$.currentTarget.value;
      filteredOptions = this.filterOptions(value);
      return this.setState({
        focusedOption: filteredOptions.length === 1 || typeof (filteredOptions != null ? (ref$ = filteredOptions[0]) != null ? ref$.newOption : void 8 : void 8) === 'undefined' ? 0 : 1,
        open: this.state.open || value.length > 0,
        search: value
      });
    },
    isBelowLimit: function(props){
      var ref$, maxItems, values;
      if ((ref$ = props || this.props) != null) {
        maxItems = ref$.maxItems, values = ref$.values;
      }
      return typeof maxItems === 'undefined' || values.length < maxItems;
    },
    showOptions: function(show, props){
      var ref$, disabled, multi, options, values;
      ref$ = props || this.props, disabled = ref$.disabled, multi = ref$.multi, options = ref$.options, values = ref$.values;
      return {
        open: (show && !multi) || (show && this.isBelowLimit(props) && !disabled && values.length < options.length)
      };
    }
  });
  function import$(obj, src){
    var own = {}.hasOwnProperty;
    for (var key in src) if (own.call(src, key)) obj[key] = src[key];
    return obj;
  }
  function in$(x, xs){
    var i = -1, l = xs.length >>> 0;
    while (++i < l) if (x === xs[i]) return true;
    return false;
  }
}).call(this);
